@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco.h,v 1.514 2014/07/07 06:04:22 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco.h,v 1.518 2014/09/25 05:12:09 zender Exp $ */
 
 /* Purpose: netCDF Operator (NCO) definitions */
 
@@ -13,6 +13,7 @@
 #ifndef NCO_H /* Contents have not yet been inserted in current source file */
 #define NCO_H
 
+/* Token added by Pedro to help debugging in nco_grp_*.c */
 #define DEBUG_LEAKS
 
 /* Standard header files */
@@ -20,6 +21,9 @@
 
 /* 3rd party vendors */
 #include <netcdf.h> /* netCDF definitions and C library */
+#ifdef NC_HAVE_META_H
+# include <netcdf_meta.h> /* NC_VERSION_..., NC_HAVE_RENAME_GRP */
+#endif /* !NC_HAVE_META_H */
 
 /* Personal headers */
 #include "nco_typ.h" /* Type definitions, opaque types */
@@ -241,6 +245,22 @@ extern "C" {
 /* Compatibility tokens for when NCO compiled with older netcdf.h
    It is hard to track where/when many tokens defined
    Easiest to individually check for pre-definition of each */
+
+  /* 20140924: netCDF meta-information available in library versions 4.3.3-rc2 
+     Define compatibility tokens for earlier libraries equivalent to 3.6.0 */
+#ifndef NC_VERSION_MAJOR
+ #define NC_VERSION_MAJOR 3
+#endif /* !NC_VERSION_MAJOR */
+#ifndef NC_VERSION_MINOR
+ #define NC_VERSION_MINOR 6
+#endif /* !NC_VERSION_MINOR */
+#ifndef NC_VERSION_PATCH
+ #define NC_VERSION_PATCH 0
+#endif /* !NC_VERSION_PATCH */
+#ifndef NCO_NC_LIB_VERSION
+# define NCO_NC_LIB_VERSION ( NC_VERSION_MAJOR * 100 + NC_VERSION_MINOR * 10 + NC_VERSION_PATCH )
+#endif /* !NCO_NC_LIB_VERSION */
+
 /* Compatibility tokens new to netCDF4 netcdf.h: */
 #ifndef NC_NETCDF4
 # define NC_NETCDF4  0x1000 /* Use netCDF-4/HDF5 format. Mode flag for nc_create(). */
@@ -536,8 +556,8 @@ extern "C" {
   }; /* end nco_mrd_cnv */
 
   enum nco_rth_cnv{ /* [enm] Arithmetic convention to assume */
-    nco_rth_flt_flt, /* 0 Keep single-precision floating point (default and NCO historical norm) */
-    nco_rth_flt_dbl  /* 1 Promote single-precision floating point to double before arithmetic */
+    nco_rth_flt_flt, /* 0 Keep single-precision floating point (NCO default through version 4.3.5 20130927) */
+    nco_rth_flt_dbl  /* 1 Promote single-precision floating point to double before arithmetic (NCO default since version 4.3.6 20130927)*/
   }; /* end nco_rth_cnv */
 
   enum nco_upk_cnv{ /* [enm] Unpacking convention to assume */
