@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnk.c,v 1.128 2014/08/22 01:01:21 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnk.c,v 1.129 2014/09/25 04:41:15 zender Exp $ */
 
 /* Purpose: NCO utilities for chunking */
 
@@ -908,7 +908,7 @@ nco_cnk_sz_set_trv /* [fnc] Set chunksize parameters (GTT version of nco_cnk_sz_
   if(is_rec_var || is_chk_var || is_cmp_var) must_be_chunked=True; else must_be_chunked=False;
 
   /* Is variable currently chunked? */
-  if(nco_fmt_xtn_get() != nco_fmt_xtn_hdf4) is_chunked=nco_cnk_dsk_inq(grp_id_in,var_id_in);
+  if(nco_fmt_xtn_get() != nco_fmt_xtn_hdf4 || NCO_NC_LIB_VERSION >= 433) is_chunked=nco_cnk_dsk_inq(grp_id_in,var_id_in);
 
   /* Does this variable have any user-chunked dimensions? */
   if(cnk_plc == nco_cnk_plc_xpl){
