@@ -1,10 +1,11 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnk.h,v 1.37 2014/02/01 02:52:30 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnk.h,v 1.39 2014/08/14 22:44:39 zender Exp $ */
 
 /* Purpose: Description (definition) of chunking functions */
 
 /* Copyright (C) 1995--2014 Charlie Zender
-   License: GNU General Public License (GPL) Version 3
-   See http://www.gnu.org/copyleft/gpl.html for full license text */
+   This file is part of NCO, the netCDF Operators. NCO is free software.
+   You can redistribute and/or modify NCO under the terms of the 
+   GNU General Public License (GPL) Version 3 with exceptions described in the LICENSE file */
 
 /* Usage:
    #include "nco_cnk.h" *//* Chunking */
@@ -36,7 +37,8 @@ enum nco_cnk_plc{ /* [enm] Chunking policy */
   nco_cnk_plc_g3d, /* 3 [enm] Chunk variables >= three dimensions */
   nco_cnk_plc_xpl, /* 4 [enm] Chunk variables with explicitly chunked dimensions */
   nco_cnk_plc_xst, /* 5 [enm] Chunk variables that are chunked in input */
-  nco_cnk_plc_uck /* 6 [enm] Unchunk all chunked variables */
+  nco_cnk_plc_uck, /* 6 [enm] Unchunk all chunked variables */
+  nco_cnk_plc_r1d /* 7 [enm] Chunk record coordinates == one dimension */
 }; /* end nco_cnk_plc enum */
 
 /* Chunking type maps: */
