@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_grp_utl.h,v 1.514 2014/08/26 17:53:43 pvicente Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_grp_utl.h,v 1.516 2014/09/23 18:43:06 zender Exp $ */
 
 /* Purpose: Group utilities */
 
@@ -567,12 +567,11 @@ nco_bld_lmt                           /* [fnc] Assign user specified dimension l
  nco_bool FORTRAN_IDX_CNV,            /* I [flg] Hyperslab indices obey Fortran convention */
  trv_tbl_sct * const trv_tbl);        /* I/O [sct] Traversal table */
 
-
 void 
-nco_msa_var_get_lmn_trv             /* [fnc] Read a used defined limit */
+nco_msa_var_get_rec_trv             /* [fnc] Read one record of a variable */
 (const int nc_id,                   /* I [ID] netCDF file ID */
  var_sct *var_prc,                  /* I/O [sct] Variable */
- const char * const rec_nm_fll,     /* I [sng] Full name of record being done in loop (trv_tbl->lmt_rec[idx_rec]->nm_fll ) */
+ const char * const rec_nm_fll,     /* I [sng] Record dimension name */
  const long idx_rec_crr_in,         /* [idx] Index of current record in current input file */
  const trv_tbl_sct * const trv_tbl);/* I [sct] GTT (Group Traversal Table) */
 
@@ -589,18 +588,6 @@ nco_var_get_wgt_trv                   /* [fnc] Retrieve weighting or mask variab
  const var_sct * const var,           /* I [sct] Variable that needs the weight/mask variable */
  const trv_tbl_sct * const trv_tbl);  /* I [lst] Traversal table */
 
-dmn_trv_sct *                         /* O [sct] Table dimension object */
-nco_dmn_usr_sng                       /* [fnc] Parse input string and return table dimension object */
-(const char * const usr_sng,          /* I [sng] Object name */
- const trv_tbl_sct * const trv_tbl,   /* I [lst] Traversal table */ 
- nco_bool *is_opt);                   /* O [flg] Dimension presence is optional (name has '.') */       
-    
-trv_sct *                             /* O [sct] Table object */
-nco_obj_usr_sng                       /* [fnc] Parse input string and return table object */
-(const char * const usr_sng,          /* I [sng] Object name */
- const trv_tbl_sct * const trv_tbl,   /* I [lst] Traversal table */ 
- nco_bool *is_opt);                   /* O [flg] Dimension presence is optional (input string has '.') */
-
 void                                  
 nco_aed_prc_grp                       /* [fnc] Process attributes in groups */
 (const int nc_id,                     /* I [id] netCDF file ID */
