@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_msa.c,v 1.250 2014/08/27 04:44:01 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_msa.c,v 1.266 2014/10/01 00:08:18 zender Exp $ */
 
 /* Purpose: Multi-slabbing algorithm */
 
@@ -19,6 +19,9 @@ nco_msa_rcr_clc /* [fnc] Multi-slab algorithm (recursive routine, returns a sing
 {
   /* NB: nco_msa_rcr_clc() with same nc_id contains OpenMP critical region */
   /* Purpose: Multi-slab algorithm (recursive routine, returns a single slab pointer) */
+
+  const char fnc_nm[]="nco_msa_rcr_clc()"; /* [sng] Function name */
+
   int idx;
   int nbr_slb;
   void *vp;
@@ -65,7 +68,9 @@ nco_msa_rcr_clc /* [fnc] Multi-slab algorithm (recursive routine, returns a sing
 
     for(idx=0;idx<dpt_crr_max;idx++) var_sz*=(idx<dpt_crr ? lmt[idx]->cnt : lmt_lst[idx]->dmn_cnt);
 
-    vp=(void *)nco_calloc((size_t)var_sz,nco_typ_lng(vara->type));
+    /* Used nco_callloc() for unknown reasons until 20140930 */
+    /*    vp=(void *)nco_calloc((size_t)var_sz,nco_typ_lng(vara->type));*/
+    vp=(void *)nco_malloc(var_sz*nco_typ_lng(vara->type));
 
     lcnt=nco_typ_lng(vara->type);
     for(idx=dpt_crr+1;idx<dpt_crr_max;idx++) lcnt*=lmt_lst[idx]->dmn_cnt;
@@ -92,7 +97,7 @@ nco_msa_rcr_clc /* [fnc] Multi-slab algorithm (recursive routine, returns a sing
         } /* end while */
         cp_fst+=slb_sz;      
       } /* end loop over two slabs */
-    } else { 
+    }else{ 
       /* Multiple hyper-slabs */
       while(nco_msa_clc_idx(True,lmt_lst[dpt_crr],&indices[0],&lmt_ret,&slb_idx)){
         cp_stp=(char *)vp+cp_fst;
@@ -120,6 +125,7 @@ nco_msa_rcr_clc /* [fnc] Multi-slab algorithm (recursive routine, returns a sing
 
 read_lbl:
   { 
+    int dmn_srd_nbr=0;
     long var_sz=1L;
     long srd_prd=1L; /* Product of strides */
     long *dmn_srt;
@@ -136,6 +142,7 @@ read_lbl:
       dmn_srd[idx]=lmt[idx]->srd;
       var_sz*=dmn_cnt[idx];
       srd_prd*=lmt[idx]->srd;
+      if(lmt[idx]->srd > 1L) dmn_srd_nbr++;
     } /* end loop over idx */
 
     vp=(void *)nco_malloc(var_sz*nco_typ_lng(vara->type));
@@ -143,13 +150,76 @@ read_lbl:
     /* Block is critical/thread-safe for identical/distinct in_id's */
     { /* begin potential OpenMP critical */
       /* Check for stride */
-      if(var_sz > 0){ /* Allow for zero-size record variables TODO nco711 */
+      if(var_sz > 0L){ /* Allow for zero-size record variables TODO nco711 */
 	/* 20140304: User reported extreme slowdown accessing 3-D data with strides across DAP connection
 	   Info from R. Signell and J. Whitaker suggests using nco_get_vars() instead of nco_get_varm() may be crucial
 	   Tests bear this out: get_varm() is tortuously slow across DAP 
 	   Always use get_vars() for strides unless using full get_varm() mapping vector */
 	/*          (void)nco_get_varm(vara->nc_id,vara->id,dmn_srt,dmn_cnt,dmn_srd,(long *)NULL,vp,vara->type); */
-        if(srd_prd == 1L) (void)nco_get_vara(vara->nc_id,vara->id,dmn_srt,dmn_cnt,vp,vara->type); else (void)nco_get_vars(vara->nc_id,vara->id,dmn_srt,dmn_cnt,dmn_srd,vp,vara->type);
+        if(srd_prd == 1L){
+	  (void)nco_get_vara(vara->nc_id,vara->id,dmn_srt,dmn_cnt,vp,vara->type); 
+	}else{
+	  /* 20140926: nc_get_vars() performs poorly on netCDF4 files
+	     Long investigation sparked by Parker Norton on 20140718 revealed nc_get_vars() calls nc_get_vara() once 
+	     per element in the strided hyperslab. This quickly become unusable for large hyperslabs.
+	     Ultimate fix may be using HDF5 algorithm H5S_set_hyperslab() as described in Jira NCF-301
+	     Until then, should decompose a single-stride request into a loop over contiguous non-stride requests */
+
+	  int fl_in_fmt; /* [enm] Input file format */
+
+	  nco_bool USE_NC4_SRD_WORKAROUND=False; /* [flg] Use faster access for strided hyperslabs in netCDF4 files */
+
+	  (void)nco_inq_format(vara->nc_id,&fl_in_fmt);
+
+	  if((fl_in_fmt == NC_FORMAT_NETCDF4 || fl_in_fmt == NC_FORMAT_NETCDF4_CLASSIC) && (dmn_srd_nbr == 1))
+	    USE_NC4_SRD_WORKAROUND=True;
+
+	  if(!USE_NC4_SRD_WORKAROUND){
+	    if(nco_dbg_lvl_get() >= nco_dbg_var && srd_prd > 1L) (void)fprintf(stderr,"%s: INFO %s reports calling nco_get_vars() for strided hyperslab access. In case of slow response, please ask NCO developers to extend USE_NC4_SRD_WORKAROUND to handle your use-case.\n",nco_prg_nm_get(),fnc_nm);
+	    (void)nco_get_vars(vara->nc_id,vara->id,dmn_srt,dmn_cnt,dmn_srd,vp,vara->type);
+	  }else{
+	    int srd_nbr; /* [nbr] Number of strides requested in the single strided dimension (not number of strided dimensions) */
+	    int srd_idx; /* [idx] Counter for how many times to call nco_get_var() */
+	    int idx_srd; /* [idx] Index of strided dimension */
+	    int dmn_idx; /* [idx] Index for dimensions */
+	    long srd_sz=1L; /* [nbr] Size of hyperslab of one stride */
+	    char *cp_srd; /* [ptr] Use char pointer because arithmetic with void pointer is non-standard */
+
+	    if(nco_dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"%s: INFO %s using USE_NC4_SRD_WORKAROUND for faster access to strided hyperslabs in netCDF4 datasets\n",nco_prg_nm_get(),fnc_nm);
+
+	    /* Find strided dimension */
+	    for(dmn_idx=0;dmn_idx<dpt_crr_max;dmn_idx++)
+	      if(dmn_srd[dmn_idx] != 1L) break;
+
+	    assert(dmn_idx != dpt_crr_max);
+	    idx_srd=dmn_idx;
+
+	    /* Find size of hyperslab of each stride */
+	    for(dmn_idx=0;dmn_idx<dpt_crr_max;dmn_idx++)
+	      if(dmn_idx != idx_srd) srd_sz*=dmn_cnt[dmn_idx];
+
+	    /* Number of strides within memory */
+	    srd_nbr=var_sz/srd_sz;
+	    assert(srd_nbr == dmn_cnt[idx_srd]);
+
+	    if(nco_dbg_lvl_get() >= nco_dbg_io){
+	      (void)fprintf(stderr,"%s: INFO %s USE_NC4_SRD_WORKAROUND reports variable %s has dmn_nbr = %d, dpt_crr_max = %d, idx_srd = %d, srd_nbr = %d, srd_sz = %ld\n",nco_prg_nm_get(),fnc_nm,vara->nm,vara->nbr_dim,dpt_crr_max,idx_srd,srd_nbr,srd_sz);
+	      (void)fprintf(stdout,"%s: %s hyperslabbing variable %s, var_sz=%ld\n",nco_prg_nm_get(),fnc_nm,vara->nm,var_sz);
+	      for(idx=0;idx<dpt_crr_max;idx++) (void)fprintf(stdout,"dmn_idx=%d srt=%ld, cnt=%ld, srd=%ld\n",idx,dmn_srt[idx],dmn_cnt[idx],dmn_srd[idx]);
+	    } /* endif dbg */
+
+	    cp_srd=(char *)vp;
+	    /* Replace multiple strides by loop over single strides */
+	    dmn_cnt[idx_srd]=1L;
+	    for(srd_idx=0;srd_idx<srd_nbr;srd_idx++){
+	      /* Point next stride on disk */
+	      if(srd_idx != 0) dmn_srt[idx_srd]+=dmn_srd[idx_srd];
+	      /* Place results in next portion of RAM */
+	      if(srd_idx != 0) cp_srd+=srd_sz*nco_typ_lng(vara->type);
+	      (void)nco_get_vara(vara->nc_id,vara->id,dmn_srt,dmn_cnt,(void *)cp_srd,vara->type);
+	    } /* end loop over srd */
+	  } /* endif workaround */
+	} /* srd_prd != 1L */
       } /* end if var_sz */
     } /* end potential OpenMP critical */
 
@@ -160,7 +230,7 @@ read_lbl:
     /* Put size into vara */
     vara->sz=var_sz;
     return vp;
-  }/* end read_lbl */
+  } /* end read_lbl */
 
 } /* end nco_msa_rcr_clc() */
 
@@ -175,11 +245,11 @@ nco_msa_prn_idx(lmt_msa_sct *lmt_i)
 
   indices=(long *)nco_malloc(size *sizeof(long));
 
-  printf("name=%s total size=%ld\n",lmt_i->dmn_nm,lmt_i->dmn_cnt);
+  (void)fprintf(stdout,"name=%s total size=%ld\n",lmt_i->dmn_nm,lmt_i->dmn_cnt);
 
   for(idx=0;idx<size;idx++) indices[idx]=lmt_i->lmt_dmn[idx]->srt;
 
-  while(nco_msa_clc_idx(False,lmt_i,&indices[0],&lmt,&slb_nbr)) printf("slb_nbr=%d, srt=%ld, end=%ld, cnt=%ld, srd=%ld\n",slb_nbr,lmt.srt,lmt.end,lmt.cnt,lmt.srd);
+  while(nco_msa_clc_idx(False,lmt_i,&indices[0],&lmt,&slb_nbr)) (void)fprintf(stdout,"slb_nbr=%d, srt=%ld, end=%ld, cnt=%ld, srd=%ld\n",slb_nbr,lmt.srt,lmt.end,lmt.cnt,lmt.srd);
 } /* end nco_msa_prn_idx() */
 
 nco_bool /* [flg] There are more limits to process in the slab */
@@ -191,19 +261,20 @@ nco_msa_clc_idx
  int *slb) /* slab which above limit refers to */ 
 {
   /* A messy, unclear, inefficient routine that needs re-writing
-  if NORMALIZE then return the slab number and the hyperslab in lmt
-  NB: This is the slab _WITHIN_ the slab so stride is ALWAYS 1 */
+     if NORMALIZE then return the slab number and the hyperslab in lmt
+     NB: This is the slab _WITHIN_ the slab so stride is ALWAYS 1 */
   int sz_idx;
   int size=lmt_a->lmt_dmn_nbr;
-  nco_bool *mnm;
-  nco_bool rcd;
-
   int prv_slb=0;
   int crr_slb=0;
+
   long crr_idx;
   long prv_idx=long_CEWI;
   long cnt=0L;
 
+  nco_bool *mnm;
+  nco_bool rcd;
+
   mnm=(nco_bool *)nco_malloc(size*sizeof(nco_bool));
 
   lmt->srt=-1L;
@@ -218,7 +289,7 @@ nco_msa_clc_idx
       if(mnm[sz_idx]){crr_slb=sz_idx;break;}
 
       if(crr_slb == -1){
-        if(lmt->srt == -1){
+        if(lmt->srt == -1L){
           rcd=False;
           goto cln_and_xit;
         }else break;
@@ -226,7 +297,7 @@ nco_msa_clc_idx
 
       if(mnm[prv_slb]) crr_slb=prv_slb;
 
-      if(lmt->srt > -1 && crr_slb != prv_slb) break;
+      if(lmt->srt > -1L && crr_slb != prv_slb) break;
 
       if(lmt->cnt > 1L){
         (lmt->cnt)++;
@@ -249,7 +320,7 @@ nco_msa_clc_idx
       for(sz_idx=0;sz_idx<size;sz_idx++){
         if(mnm[sz_idx]){
           indices[sz_idx]+=lmt_a->lmt_dmn[sz_idx]->srd;
-          if(indices[sz_idx] > lmt_a->lmt_dmn[sz_idx]->end) indices[sz_idx]=-1;
+          if(indices[sz_idx] > lmt_a->lmt_dmn[sz_idx]->end) indices[sz_idx]=-1L;
         }
       } /* end loop over sz_idx */
       prv_idx=crr_idx;
@@ -282,9 +353,8 @@ nco_msa_ram_2_dsk /* Convert hyperslab indices (in RAM) to hyperlsab indices rel
  long *dmn_sbs_dsk,
  nco_bool flg_free)
 {
-  /*  It does not really convert RAM indices to disk indices, but given a set 
-  of RAM indices, it finds the next set of disk incdices. 
-  So it only works if the indices fed to it are continuous */
+  /* Given a set of RAM indices, routine finds next set of disk indices
+     Only works if the indices fed to it are continuous */
   int idx;
   int jdx;
   int size;
@@ -302,7 +372,7 @@ nco_msa_ram_2_dsk /* Convert hyperslab indices (in RAM) to hyperlsab indices rel
         dmn_indices[idx][jdx]=lmt_msa[idx]->lmt_dmn[jdx]->srt;
       dmn_sbs_prv[idx]=-1L;
     }
-    initialize=1;
+    initialize=True;
   }
   for(idx=0;idx <nbr_dim;idx ++){
     size=lmt_msa[idx]->lmt_dmn_nbr;
@@ -335,7 +405,7 @@ nco_msa_ram_2_dsk /* Convert hyperslab indices (in RAM) to hyperlsab indices rel
     for(jdx=0;jdx<size;jdx++){
       if(mnm[jdx]){
         dmn_indices[idx][jdx]+=lmt_msa[idx]->lmt_dmn[jdx]->srd;
-        if(dmn_indices[idx][jdx] > lmt_msa[idx]->lmt_dmn[jdx]->end) dmn_indices[idx][jdx]=-1;
+        if(dmn_indices[idx][jdx] > lmt_msa[idx]->lmt_dmn[jdx]->end) dmn_indices[idx][jdx]=-1L;
       }
     } /* end for  jdx */
   } /* end for idx */ 
@@ -345,10 +415,9 @@ nco_msa_ram_2_dsk /* Convert hyperslab indices (in RAM) to hyperlsab indices rel
   /* Free static space on last call */
   if(flg_free){
     (void)nco_free(dmn_sbs_prv);
-    for(idx=0;idx <nbr_dim;idx++)
-      (void)nco_free(dmn_indices[idx]);
+    for(idx=0;idx<nbr_dim;idx++) (void)nco_free(dmn_indices[idx]);
     (void)nco_free(dmn_indices);
-    initialize=0;
+    initialize=False;
   }
 } /* end nco_msa_ram_2_dsk() */
 
@@ -383,7 +452,7 @@ nco_msa_clc_cnt
       for(idx=0;idx<size;idx++){
         if(mnm[idx]){
           indices[idx]+=lmt_lst->lmt_dmn[idx]->srd;
-          if(indices[idx] > lmt_lst->lmt_dmn[idx]->end) indices[idx]=-1;
+          if(indices[idx] > lmt_lst->lmt_dmn[idx]->end) indices[idx]=-1L;
         } /* end if */
       } /* end loop over idx */
       cnt++;
@@ -402,7 +471,7 @@ nco_msa_ovl
 (lmt_msa_sct *lmt_lst)
 {
   /* Purpose: Return true if limits overlap
-  NB: Assumes that limits have been sorted */
+     NB: Assumes that limits have been sorted */
 
   long idx;
   long jdx;
@@ -412,9 +481,9 @@ nco_msa_ovl
   /* defererence */
   lmt=lmt_lst->lmt_dmn;
 
-  for(idx=0; idx<sz; idx++)
-    for(jdx=idx+1; jdx<sz ;jdx++)
-      if( lmt[jdx]->srt <= lmt[idx]->end) return True;  
+  for(idx=0L;idx<sz;idx++)
+    for(jdx=idx+1L;jdx<sz;jdx++)
+      if(lmt[jdx]->srt <= lmt[idx]->end) return True;  
 
   return False;
 } /* end nco_msa_ovl() */
@@ -457,10 +526,10 @@ nco_msa_min_idx /* [fnc] Find minimum values in current and return minimum value
   long min_val=LONG_MAX;
 
   for(sz_idx=0;sz_idx<size;sz_idx++)
-    if(current[sz_idx] != -1 && current[sz_idx] < min_val) min_val=current[sz_idx];
+    if(current[sz_idx] != -1L && current[sz_idx] < min_val) min_val=current[sz_idx];
 
   for(sz_idx=0;sz_idx<size;sz_idx++)
-    mnm[sz_idx]=((current[sz_idx] != -1 && current[sz_idx] == min_val) ? True : False);
+    mnm[sz_idx]=((current[sz_idx] != -1L && current[sz_idx] == min_val) ? True : False);
 
   return min_val;
 } /* end nco_msa_min_idx() */
@@ -480,11 +549,11 @@ nco_msa_var_get    /* [fnc] Get variable data from disk taking account of multih
   lmt_msa_sct **lmt_msa;
   lmt_sct **lmt;
 
-  nbr_dim=var_in->nbr_dim;	
+  nbr_dim=var_in->nbr_dim;
 
   /* Refresh nc_id with in_id, NB: makes OpenMP threading work
-  Should have been included in release 3.9.5
-  Fixes TODO nco956 */
+     Should have been included in release 3.9.5
+     Fixes TODO nco956 */
   var_in->nc_id=in_id; 
 
   /* Scalars */
@@ -521,9 +590,7 @@ nco_msa_var_get    /* [fnc] Get variable data from disk taking account of multih
 
 do_upk:
   /* Following code copied from nco_var_get() */
-
   if(var_in->pck_dsk) var_in=nco_cnv_mss_val_typ(var_in,var_in->typ_dsk);
-  /*    var=nco_cnv_mss_val_typ(var,var->typ_dsk);*/
 
   /* Type of variable and missing value in memory are now same as type on disk */
   var_in->type=var_in->typ_dsk; /* [enm] Type of variable in RAM */
@@ -769,8 +836,8 @@ nco_msa_wrp_splt_trv   /* [fnc] Split wrapped dimensions (GTT version) */
 
   int idx;
   int jdx;
-  int size=dmn_trv->lmt_msa.lmt_dmn_nbr;  /* [nbr] Number of limit structures */
-  long dmn_sz_org=dmn_trv->sz;    /* [nbr] Size of dimension */
+  int size=dmn_trv->lmt_msa.lmt_dmn_nbr; /* [nbr] Number of limit structures */
+  long dmn_sz_org=dmn_trv->sz; /* [nbr] Size of dimension */
   long srt;
   long cnt;
   long srd;
@@ -785,7 +852,7 @@ nco_msa_wrp_splt_trv   /* [fnc] Split wrapped dimensions (GTT version) */
 
       lmt_wrp=(lmt_sct *)nco_malloc(2*sizeof(lmt_sct));
 
-      /* "trv": Initialize  */
+      /* "trv": Initialize */
       (void)nco_lmt_init(&lmt_wrp[0]);
       (void)nco_lmt_init(&lmt_wrp[1]);
 
@@ -1426,10 +1493,7 @@ nco_cpy_msa_lmt                     /* [fnc] Copy MSA struct from table to local
     } /* If there are limits for this variable dimension, get MSA from table */
 
     /* This dimension must have either a coordinate or a dimension pointer */
-    else
-    {
-      assert(0);
-    }
+    else assert(False);
 
   } /* Loop dimensions for object (variable)  */
 
@@ -1448,12 +1512,11 @@ nco_msa_var_get_trv                 /* [fnc] Get variable data from disk taking
 
   lmt_msa_sct **lmt_msa;
   lmt_sct **lmt;
+
+  nc_type mss_typ_tmp;
+
   trv_sct *var_trv;
 
-  nc_type typ_tmp;
-
-  void *void_ptr;
-
   /* Obtain variable GTT object using full variable name */
   var_trv=trv_tbl_var_nm_fll(var_in->nm_fll,trv_tbl);
   assert(var_trv);
@@ -1465,7 +1528,7 @@ nco_msa_var_get_trv                 /* [fnc] Get variable data from disk taking
   var_in->nc_id=grp_id; 
 
   assert(nbr_dim == var_trv->nbr_dmn);
-  assert(strcmp(var_in->nm_fll,var_trv->nm_fll) == 0);
+  assert(!strcmp(var_in->nm_fll,var_trv->nm_fll));
 
   /* Scalars */
   if(nbr_dim == 0){
@@ -1481,59 +1544,55 @@ nco_msa_var_get_trv                 /* [fnc] Get variable data from disk taking
   /* Copy from table to local MSA */
   (void)nco_cpy_msa_lmt(var_trv,&lmt_msa);
 
-
   if(nco_dbg_lvl_get() == nco_dbg_old){
     (void)fprintf(stdout,"%s: DEBUG %s reading <%s>\n",nco_prg_nm_get(),fnc_nm,var_trv->nm_fll);
-    /* Loop dimensions */
     for(int idx_dmn=0;idx_dmn<var_trv->nbr_dmn;idx_dmn++){
-      (void)fprintf(stdout,"%s: DEBUG %s <%s> elements %ld",nco_prg_nm_get(),fnc_nm,
-        lmt_msa[idx_dmn]->dmn_nm,lmt_msa[idx_dmn]->dmn_cnt);
-      for(int idx_lmt=0;idx_lmt<lmt_msa[idx_dmn]->lmt_dmn_nbr;idx_lmt++){
-        (void)fprintf(stdout," : %ld (%ld->%ld)",lmt_msa[idx_dmn]->lmt_dmn[idx_lmt]->cnt,
-          lmt_msa[idx_dmn]->lmt_dmn[idx_lmt]->srt,
-          lmt_msa[idx_dmn]->lmt_dmn[idx_lmt]->end);
-      }
+      (void)fprintf(stdout,"%s: DEBUG %s <%s> elements %ld",nco_prg_nm_get(),fnc_nm,lmt_msa[idx_dmn]->dmn_nm,lmt_msa[idx_dmn]->dmn_cnt);
+      for(int idx_lmt=0;idx_lmt<lmt_msa[idx_dmn]->lmt_dmn_nbr;idx_lmt++) (void)fprintf(stdout," : %ld (%ld->%ld)",lmt_msa[idx_dmn]->lmt_dmn[idx_lmt]->cnt,lmt_msa[idx_dmn]->lmt_dmn[idx_lmt]->srt,lmt_msa[idx_dmn]->lmt_dmn[idx_lmt]->end);
       (void)fprintf(stdout,"\n");
-    } /* Loop dimensions */
-  }
+    } /* end loop over dimensions */
+  } /* endif dbg */
 
+  /* Call super-dooper recursive routine
+     nco_msa_rcr_clc requires that var_in->type be on-disk type
+     Could replace var->type by var->typ_dsk in nco_msa_rcr_clc() but that seems inelegant 
+     Instead, risk putting val and mss_val types briefly out-of-sync by pretending var->type is typ_dsk
+     Save current type of missing value in RAM in temporary variable and conform new variable to that below
+     20140930: This is (too?) confusing and hard-to-follow, a better solution is to add a field mss_val_typ 
+     to var_sct and then separately and explicitly track types of both val and mss_val members. */
+  mss_typ_tmp=var_in->type;
+  var_in->type=var_in->typ_dsk;
+  var_in->val.vp=nco_msa_rcr_clc((int)0,nbr_dim,lmt,lmt_msa,var_in);
+  var_in->type=mss_typ_tmp;
 
-  /* Call super-dooper recursive routine */
-  typ_tmp=var_in->type;
-  var_in->type=var_in->typ_dsk; 
-  void_ptr=nco_msa_rcr_clc((int)0,nbr_dim,lmt,lmt_msa,var_in);
-
-  var_in->type=typ_tmp;
-  var_in->val.vp=void_ptr;
-
-  /* Free  */
+  /* Free */
   (void)nco_lmt_msa_free(var_trv->nbr_dmn,lmt_msa);
   lmt=(lmt_sct **)nco_free(lmt);
 
 do_upk:
-  /* Following code copied from nco_var_get() */
-
-  if(var_in->pck_dsk) var_in=nco_cnv_mss_val_typ(var_in,var_in->typ_dsk);
+  /* Missing value type synchronization:
+     Avoid re-reading missing value every ncra record by converting input value to disk type
+     var_in->type still reflects missing value type, not variable value type */
+  if(var_in->pck_dsk && (mss_typ_tmp != var_in->typ_dsk)) var_in=nco_cnv_mss_val_typ(var_in,var_in->typ_dsk);
+  var_in->type=var_in->typ_dsk;
 
   /* Type of variable and missing value in memory are now same as type on disk */
-  var_in->type=var_in->typ_dsk; /* [enm] Type of variable in RAM */
 
   /* Packing in RAM is now same as packing on disk pck_dbg 
-  fxm: This nco_pck_dsk_inq() call is never necessary for non-packed variables */
+     fxm: This nco_pck_dsk_inq() call is never necessary for non-packed variables */
   (void)nco_pck_dsk_inq(grp_id,var_in);
 
   /* Packing/Unpacking */
   if(nco_is_rth_opr(nco_prg_id_get())){
     /* Arithmetic operators must unpack variables before performing arithmetic
-    Otherwise arithmetic will produce garbage results */
+       Otherwise arithmetic will produce garbage results */
     /* 20050519: Not sure why I originally made nco_var_upk() call SMP-critical
-    20050629: Making this region multi-threaded causes no problems */
+       20050629: Making this region multi-threaded causes no problems */
     if(var_in->pck_dsk) var_in=nco_var_upk(var_in);
   } /* endif arithmetic operator */
 
   return;
-} /* nco_msa_var_get_trv() */
-
+} /* end nco_msa_var_get_trv() */
 
 void
 nco_lmt_msa_free                    /* [fnc] Free MSA */
@@ -1541,14 +1600,12 @@ nco_lmt_msa_free                    /* [fnc] Free MSA */
  lmt_msa_sct **lmt_msa)             /* I [sct] MSA */
 {
   /* Free  */
-  for(int idx_dmn=0;idx_dmn<nbr_dmn;idx_dmn++) {
+  for(int idx_dmn=0;idx_dmn<nbr_dmn;idx_dmn++){
     lmt_msa[idx_dmn]->dmn_nm=(char *)nco_free(lmt_msa[idx_dmn]->dmn_nm);
-    for(int lmt_idx=0;lmt_idx<lmt_msa[idx_dmn]->lmt_dmn_nbr;lmt_idx++){
-      lmt_msa[idx_dmn]->lmt_dmn[lmt_idx]=nco_lmt_free(lmt_msa[idx_dmn]->lmt_dmn[lmt_idx]);
-    }
+    for(int lmt_idx=0;lmt_idx<lmt_msa[idx_dmn]->lmt_dmn_nbr;lmt_idx++) lmt_msa[idx_dmn]->lmt_dmn[lmt_idx]=nco_lmt_free(lmt_msa[idx_dmn]->lmt_dmn[lmt_idx]);
     lmt_msa[idx_dmn]->lmt_dmn=(lmt_sct **)nco_free(lmt_msa[idx_dmn]->lmt_dmn);
     lmt_msa[idx_dmn]=(lmt_msa_sct *)nco_free(lmt_msa[idx_dmn]);
   }
   lmt_msa=(lmt_msa_sct **)nco_free(lmt_msa);
-}
+} /* end nco_lmt_msa_free() */
 
