@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_netcdf.h,v 1.120 2014/07/10 23:36:22 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_netcdf.h,v 1.123 2014/09/25 05:12:09 zender Exp $ */
 
 /* Purpose: NCO wrappers for netCDF C library */
 
@@ -48,6 +48,9 @@
 
 /* 3rd party vendors */
 #include <netcdf.h> /* netCDF definitions and C library */
+#ifdef NC_HAVE_META_H
+# include <netcdf_meta.h> /* NC_VERSION_..., NC_HAVE_RENAME_GRP */	 
+#endif /* !NC_HAVE_META_H */
 #ifdef ENABLE_MPI
 # include <mpi.h> /* MPI definitions */
 # include <netcdf_par.h> /* Parallel netCDF definitions */
@@ -127,6 +130,12 @@
 # define NC_FORMAT_64BIT_DATA	NC_FORMAT_CDF5
 #endif
 
+/* netcdf.h added tokens specifically for DAP in 2012?
+   Started annotating these error codes in nco_err_exit() in NCO 4.4.6 in 201409 */
+#ifndef NC_EIO
+# define NC_EIO          (-68)      /**< Generic IO error */
+#endif
+
 /* Some netCDF3 stubs for netCDF4 routines need netCDF4-only return codes
    These netCDF4 tokens are never defined in netCDF3-only netcdf.h */
 #ifndef HAVE_NETCDF4_H
