@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncrename.c,v 1.198 2014/07/11 20:55:45 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncrename.c,v 1.205 2014/09/26 23:03:48 zender Exp $ */
 
 /* ncrename -- netCDF renaming operator */
 
@@ -93,7 +93,6 @@ main(int argc,char **argv)
   nco_bool FORCE_APPEND=False; /* Option A */
   nco_bool FORCE_OVERWRITE=False; /* Option O */
   nco_bool HISTORY_APPEND=True; /* Option h */
-  nco_bool IS_GLB_GRP_ATT=False; /* [flg] Attribute is Global or Group attribute */
   nco_bool FL_OUT_NEW=False;
   nco_bool RAM_OPEN=False; /* [flg] Open (netCDF3-only) file(s) in RAM */
   nco_bool RM_RMT_FL_PST_PRC=True; /* Option R */
@@ -114,13 +113,13 @@ main(int argc,char **argv)
   char *var_rnm_arg[NC_MAX_VARS];
   char trv_pth[]="/"; /* [sng] Root path of traversal tree */
 
-  char var_nm[NC_MAX_NAME+1];
+  char *obj_nm=NULL;
 
-  const char * const CVS_Id="$Id: ncrename.c,v 1.198 2014/07/11 20:55:45 zender Exp $"; 
-  const char * const CVS_Revision="$Revision: 1.198 $";
+  const char * const CVS_Id="$Id: ncrename.c,v 1.205 2014/09/26 23:03:48 zender Exp $"; 
+  const char * const CVS_Revision="$Revision: 1.205 $";
   const char * const opt_sht_lst="a:D:d:g:hl:Oo:p:rv:-:";
   const char dlm_chr='@'; /* Character delimiting variable from attribute name  */
-  const char opt_chr='.'; /* Character indicating presence of following variable/dimension/attribute in file is optional */
+  const char opt_chr='.'; /* Character indicating presence of following variable/dimension/attribute/group in file is optional */
 #if defined(__cplusplus) || defined(PGI_CC)
   ddra_info_sct ddra_info;
   ddra_info.flg_ddra=False;
@@ -133,6 +132,10 @@ main(int argc,char **argv)
 
   int abb_arg_nbr=0;
   int fl_nbr=0;
+  int mch_nbr_att=0; /* [nbr] Number of renamed attributes */
+  int mch_nbr_var=0; /* [nbr] Number of renamed variables */
+  int mch_nbr_grp=0; /* [nbr] Number of renamed groups */
+  int mch_nbr_dmn=0; /* [nbr] Number of renamed dimensions */
   int md_open; /* [enm] Mode flag for nc_open() call */
   int nbr_att_rnm=0; /* Option a. NB: nbr_att_rnm gets incremented */
   int nbr_dmn_rnm=0; /* Option d. NB: nbr_dmn_rnm gets incremented */
@@ -370,231 +373,238 @@ main(int argc,char **argv)
   /* Construct GTT (Group Traversal Table), check -v and -g input names and create extraction list*/
   (void)nco_bld_trv_tbl(nc_id,trv_pth,(int)0,NULL,(int)0,NULL,False,False,NULL,(int)0,NULL,(int) 0,False,False,False,False,True,nco_pck_plc_nil,NULL,trv_tbl);
 
-  /* Loop input variable names */
+  /* Rename variables */
   for(int idx_var=0;idx_var<nbr_var_rnm;idx_var++){
-    trv_sct *obj_trv=NULL; /* [sct] Table object */
-    nco_bool is_opt;       /* [flg] Presence is optional (name has '.') */
+    int mch_nbr=0; /* [nbr] Number of matching objects */
+    nco_bool is_opt=False; /* [flg] Presence is optional (name has '.') */
+    size_t mch_fst=0L; /* [nbr] Number of characters by which to offset string comparison */
 
-    /* Inquire if any object matches  */
-    obj_trv=nco_obj_usr_sng(var_rnm_lst[idx_var].old_nm,trv_tbl,&is_opt);
+    /* Initialize */
+    if(var_rnm_lst[idx_var].old_nm[0] == opt_chr){
+      is_opt=True;
+      mch_fst=1L;
+    } /* end if */
 
-    /* Optional case with variable found */
-    if(obj_trv && obj_trv->nco_typ == nco_obj_typ_var && is_opt == True){
-      (void)nco_inq_grp_full_ncid(nc_id,obj_trv->grp_nm_fll,&grp_id);
-      /* Use the pair group ID/relative object name found (instead of var_rnm_lst[idx_var].old_nm)  */
-      rcd=nco_inq_varid(grp_id,obj_trv->nm,&var_rnm_lst[idx_var].id);
-      (void)nco_rename_var(grp_id,var_rnm_lst[idx_var].id,var_rnm_lst[idx_var].new_nm);
-      if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed variable \'%s\' to \'%s\'\n",nco_prg_nm,var_rnm_lst[idx_var].old_nm+1L,var_rnm_lst[idx_var].new_nm);
-      /* Optional case with no object found */
-    }else if (obj_trv == NULL) {
-      (void)fprintf(stdout,"%s: WARNING Variable \'%s\' not present in %s, skipping it.\n",nco_prg_nm,var_rnm_lst[idx_var].old_nm+1L,fl_in);
-      /* Reset error code */
-      rcd=NC_NOERR;
-      /* Variable name does not contain opt_chr so variable presence is required */
-    }else if (obj_trv && obj_trv->nco_typ == nco_obj_typ_var && is_opt == False){ 
-      (void)nco_inq_grp_full_ncid(nc_id,obj_trv->grp_nm_fll,&grp_id);
-      /* Use the pair group ID/relative object name found (instead of var_rnm_lst[idx_var].old_nm)  */
-      rcd=nco_inq_varid(grp_id,obj_trv->nm,&var_rnm_lst[idx_var].id);
-      (void)nco_rename_var(grp_id,var_rnm_lst[idx_var].id,var_rnm_lst[idx_var].new_nm);
-      if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed variable \'%s\' to \'%s\'\n",nco_prg_nm,var_rnm_lst[idx_var].old_nm,var_rnm_lst[idx_var].new_nm);
-    } /* Variable name does not contain opt_chr so variable presence is required */
-  } /* Loop input variable names */
+    /* Compare each variable in user list to each variable name in file */
+    for(unsigned int tbl_idx=0;tbl_idx<trv_tbl->nbr;tbl_idx++)
+      if(trv_tbl->lst[tbl_idx].nco_typ == nco_obj_typ_var)
+	if(!strcmp(var_rnm_lst[idx_var].old_nm+mch_fst,trv_tbl->lst[tbl_idx].nm_fll) || !strcmp(var_rnm_lst[idx_var].old_nm+mch_fst,trv_tbl->lst[tbl_idx].nm)){
+	    mch_nbr++;
+	    (void)nco_inq_grp_full_ncid(nc_id,trv_tbl->lst[tbl_idx].grp_nm_fll,&grp_id);
+	    rcd=nco_inq_varid(grp_id,trv_tbl->lst[tbl_idx].nm,&var_rnm_lst[idx_var].id);
+	    (void)nco_rename_var(grp_id,var_rnm_lst[idx_var].id,var_rnm_lst[idx_var].new_nm);
+	    if(nco_dbg_lvl >= nco_dbg_fl) (void)fprintf(stdout,"%s: Renamed variable \'%s\' to \'%s\'\n",nco_prg_nm,trv_tbl->lst[tbl_idx].nm_fll,var_rnm_lst[idx_var].new_nm);
+	} /* endif */
 
-  /* Loop input group names */
+    if(mch_nbr == 0 && !is_opt){
+      (void)fprintf(stdout,"%s: ERROR Required variable \'%s\' is not present in input file. HINT: If presence is intended to be optional, then prefix old variable name with the period character \'%c\', i.e., \'%s -v %c%s,%s\'. With this syntax %s would succeed even if no such variable is in the file.\n",nco_prg_nm_get(),var_rnm_lst[idx_var].old_nm,opt_chr,nco_prg_nm_get(),opt_chr,var_rnm_lst[idx_var].old_nm,var_rnm_lst[idx_var].new_nm,nco_prg_nm_get());
+      nco_exit(EXIT_FAILURE);
+    }else if(mch_nbr == 0 && is_opt){
+      (void)fprintf(stdout,"%s: INFO Optional variable \'%s\' not present in %s, skipping it\n",nco_prg_nm,var_rnm_lst[idx_var].old_nm+1L,fl_in);
+    }else{
+      if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed %d variable%s from \'%s\' to \'%s\'\n",nco_prg_nm,mch_nbr,mch_nbr != 1 ? "s" : "",var_rnm_lst[idx_var].old_nm,var_rnm_lst[idx_var].new_nm);
+    } /* end else */
+
+    mch_nbr_var+=mch_nbr;
+
+  } /* end variables */
+
+  /* Rename groups */
   for(int idx_grp=0;idx_grp<nbr_grp_rnm;idx_grp++){
-    trv_sct *obj_trv=NULL; /* [sct] Table object */
-    nco_bool is_opt;       /* [flg] Presence is optional (name has '.') */
+    int mch_nbr=0; /* [nbr] Number of matching objects */
+    nco_bool is_opt=False; /* [flg] Presence is optional (name has '.') */
+    size_t mch_fst=0L; /* [nbr] Number of characters by which to offset string comparison */
 
-    /* Inquire if any object matches  */
-    obj_trv=nco_obj_usr_sng(grp_rnm_lst[idx_grp].old_nm,trv_tbl,&is_opt);
+    /* Initialize */
+    if(grp_rnm_lst[idx_grp].old_nm[0] == opt_chr){
+      is_opt=True;
+      mch_fst=1L;
+    } /* end if */
 
-    /* Optional case with group found */
-    if(obj_trv && obj_trv->nco_typ == nco_obj_typ_grp && is_opt == True){
-      (void)nco_inq_grp_full_ncid(nc_id,obj_trv->grp_nm_fll,&grp_id);
-      (void)nco_rename_grp(grp_id,grp_rnm_lst[idx_grp].new_nm);
-      if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed group \'%s\' to \'%s\'\n",nco_prg_nm,grp_rnm_lst[idx_grp].old_nm+1L,grp_rnm_lst[idx_grp].new_nm);
-      /* Optional case with no object found */
-    }else if (obj_trv == NULL){
-      (void)fprintf(stdout,"%s: WARNING Group \'%s\' not present in %s, skipping it.\n",nco_prg_nm,grp_rnm_lst[idx_grp].old_nm+1L,fl_in);
-      /* Reset error code */
-      rcd=NC_NOERR; 
-      /* Group name does not contain opt_chr so group presence is required */
-    }else if (obj_trv && obj_trv->nco_typ == nco_obj_typ_grp && is_opt == False){  
-      (void)nco_inq_grp_full_ncid(nc_id,obj_trv->grp_nm_fll,&grp_id);
-      (void)nco_rename_grp(grp_id,grp_rnm_lst[idx_grp].new_nm);
-      if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed group \'%s\' to \'%s\'\n",nco_prg_nm,grp_rnm_lst[idx_grp].old_nm,grp_rnm_lst[idx_grp].new_nm);
-    } /* Group name does not contain opt_chr so group presence is required */
-  } /* Loop input group names */
+    /* Compare each group in user list to each group name in file */
+    for(unsigned int tbl_idx=0;tbl_idx<trv_tbl->nbr;tbl_idx++)
+      if(trv_tbl->lst[tbl_idx].nco_typ == nco_obj_typ_grp)
+	if(!strcmp(grp_rnm_lst[idx_grp].old_nm+mch_fst,trv_tbl->lst[tbl_idx].grp_nm_fll) || !strcmp(grp_rnm_lst[idx_grp].old_nm+mch_fst,trv_tbl->lst[tbl_idx].grp_nm)){
+	    mch_nbr++;
+	    (void)nco_inq_grp_full_ncid(nc_id,trv_tbl->lst[tbl_idx].grp_nm_fll,&grp_id);
+	    (void)nco_rename_grp(grp_id,grp_rnm_lst[idx_grp].new_nm);
+	    if(nco_dbg_lvl >= nco_dbg_fl) (void)fprintf(stdout,"%s: Renamed group \'%s\' to \'%s\'\n",nco_prg_nm,trv_tbl->lst[tbl_idx].grp_nm_fll,grp_rnm_lst[idx_grp].new_nm);
+	} /* endif */
 
+    if(mch_nbr == 0 && !is_opt){
+      (void)fprintf(stdout,"%s: ERROR Required group \'%s\' is not present in input file. HINT: If presence is intended to be optional, then prefix old group name with the period character \'%c\', i.e., \'%s -g %c%s,%s\'. With this syntax %s would succeed even if no such group is in the file.\n",nco_prg_nm_get(),grp_rnm_lst[idx_grp].old_nm,opt_chr,nco_prg_nm_get(),opt_chr,grp_rnm_lst[idx_grp].old_nm,grp_rnm_lst[idx_grp].new_nm,nco_prg_nm_get());
+      nco_exit(EXIT_FAILURE);
+    }else if(mch_nbr == 0 && is_opt){
+      (void)fprintf(stdout,"%s: INFO Optional group \'%s\' not present in %s, skipping it\n",nco_prg_nm,grp_rnm_lst[idx_grp].old_nm+1L,fl_in);
+    }else{
+      if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed %d group%s from \'%s\' to \'%s\'\n",nco_prg_nm,mch_nbr,mch_nbr != 1 ? "s" : "",grp_rnm_lst[idx_grp].old_nm,grp_rnm_lst[idx_grp].new_nm);
+    } /* end else */
 
-  /* Loop input dimension names */
+    mch_nbr_grp+=mch_nbr;
+
+  } /* end groups */
+
+  /* Rename dimensions */
   for(int idx_dmn=0;idx_dmn<nbr_dmn_rnm;idx_dmn++){
-    dmn_trv_sct *dmn_trv=NULL; /* [sct] Table dimension object */
-    nco_bool is_opt;           /* [flg] Dimension presence is optional (name has '.') */
+    int mch_nbr=0; /* [nbr] Number of matching objects */
+    nco_bool is_opt=False; /* [flg] Presence is optional (name has '.') */
+    size_t mch_fst=0L; /* [nbr] Number of characters by which to offset string comparison */
 
-    /* Inquire if any dimension matches  */
-    dmn_trv=nco_dmn_usr_sng(dmn_rnm_lst[idx_dmn].old_nm,trv_tbl,&is_opt);
+    /* Initialize */
+    if(dmn_rnm_lst[idx_dmn].old_nm[0] == opt_chr){
+      is_opt=True;
+      mch_fst=1L;
+    } /* end if */
 
-    /* Optional case with dimension found */
-    if(dmn_trv && is_opt == True){
-      (void)nco_inq_grp_full_ncid(nc_id,dmn_trv->grp_nm_fll,&grp_id);
-      /* Use the pair group ID/relative dimension name found (instead of dmn_rnm_lst[idx_dmn].old_nm)  */
-      rcd=nco_inq_dimid(grp_id,dmn_trv->nm,&dmn_rnm_lst[idx_dmn].id);
-      (void)nco_rename_dim(grp_id,dmn_rnm_lst[idx_dmn].id,dmn_rnm_lst[idx_dmn].new_nm);
-      if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed dimension \'%s\' to \'%s\'\n",nco_prg_nm,dmn_rnm_lst[idx_dmn].old_nm+1L,dmn_rnm_lst[idx_dmn].new_nm);
-      /* Optional case with no dimension found */
-    }else if (dmn_trv == NULL) {
-      (void)fprintf(stdout,"%s: WARNING Dimension \'%s\' not present in %s, skipping it.\n",nco_prg_nm,dmn_rnm_lst[idx_dmn].old_nm+1L,fl_in);
-      /* Reset error code */
-      rcd=NC_NOERR; 
-      /* ! Optional case */
-    }else if (dmn_trv && is_opt == False){
-      (void)nco_inq_grp_full_ncid(nc_id,dmn_trv->grp_nm_fll,&grp_id);
-      /* Use the pair group ID/relative dimension name found (instead of dmn_rnm_lst[idx_dmn].old_nm)  */
-      rcd=nco_inq_dimid(grp_id,dmn_trv->nm,&dmn_rnm_lst[idx_dmn].id);
-      (void)nco_rename_dim(grp_id,dmn_rnm_lst[idx_dmn].id,dmn_rnm_lst[idx_dmn].new_nm);
-      if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed dimension \'%s\' to \'%s\'\n",nco_prg_nm,dmn_rnm_lst[idx_dmn].old_nm,dmn_rnm_lst[idx_dmn].new_nm);
-    }  /* ! Optional case */
-  } /* Loop input dimension names */
+    /* Compare each dimension in user list to each dimension name in file */
+    for(unsigned int tbl_idx=0;tbl_idx<trv_tbl->nbr_dmn;tbl_idx++)
+      if(!strcmp(dmn_rnm_lst[idx_dmn].old_nm+mch_fst,trv_tbl->lst_dmn[tbl_idx].nm_fll) || !strcmp(dmn_rnm_lst[idx_dmn].old_nm+mch_fst,trv_tbl->lst_dmn[tbl_idx].nm)){
+	mch_nbr++;
+	(void)nco_inq_grp_full_ncid(nc_id,trv_tbl->lst_dmn[tbl_idx].grp_nm_fll,&grp_id);
+	rcd=nco_inq_dimid(grp_id,trv_tbl->lst_dmn[tbl_idx].nm,&dmn_rnm_lst[idx_dmn].id);
+	(void)nco_rename_dim(grp_id,dmn_rnm_lst[idx_dmn].id,dmn_rnm_lst[idx_dmn].new_nm);
+	if(nco_dbg_lvl >= nco_dbg_fl) (void)fprintf(stdout,"%s: Renamed dimension \'%s\' to \'%s\'\n",nco_prg_nm,trv_tbl->lst_dmn[tbl_idx].nm_fll,dmn_rnm_lst[idx_dmn].new_nm);
+      } /* endif */
 
-  /* Loop input attribute names */
+    if(mch_nbr == 0 && !is_opt){
+      (void)fprintf(stdout,"%s: ERROR Required dimension \'%s\' is not present in input file. HINT: If presence is intended to be optional, then prefix old dimension name with the period character \'%c\', i.e., \'%s -d %c%s,%s\'. With this syntax %s would succeed even if no such dimension is in the file.\n",nco_prg_nm_get(),dmn_rnm_lst[idx_dmn].old_nm,opt_chr,nco_prg_nm_get(),opt_chr,dmn_rnm_lst[idx_dmn].old_nm,dmn_rnm_lst[idx_dmn].new_nm,nco_prg_nm_get());
+      nco_exit(EXIT_FAILURE);
+    }else if(mch_nbr == 0 && is_opt){
+      (void)fprintf(stdout,"%s: INFO Optional dimension \'%s\' not present in %s, skipping it\n",nco_prg_nm,dmn_rnm_lst[idx_dmn].old_nm+1L,fl_in);
+    }else{
+      if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed %d dimension%s from \'%s\' to \'%s\'\n",nco_prg_nm,mch_nbr,mch_nbr != 1 ? "s" : "",dmn_rnm_lst[idx_dmn].old_nm,dmn_rnm_lst[idx_dmn].new_nm);
+    } /* end else */
+
+    mch_nbr_dmn+=mch_nbr;
+
+  } /* end dimensions */
+
+  /* Rename attributes */
+  if(nbr_att_rnm > 0) obj_nm=(char *)nco_malloc((NC_MAX_NAME+1L)*sizeof(char));
   for(int idx_att=0;idx_att<nbr_att_rnm;idx_att++){
-    int var_id;
-    int nbr_rnm=0;
+    int mch_nbr=0; /* [nbr] Number of matching objects */
     int rcd_att=0;
-    nco_bool is_opt;       /* [flg] Presence is optional (name has '.') */
-    trv_sct *trv_obj=NULL; /* [sct] Traversal object */  
+    int var_id;
+    nco_bool att_is_opt=False; /* [flg] Presence of attribute is optional (name has '.') */
+    nco_bool mch_fll_nm=False; /* [flg] User-specified object matches full name */
+    nco_bool mch_grp_all=False; /* [flg] Rename all group attributes */
+    nco_bool mch_grp_glb=False; /* [flg] Rename only global attributes */
+    nco_bool mch_obj_all=False; /* [flg] Rename all group and variable attributes */
+    nco_bool obj_is_opt=False; /* [flg] Presence of object is optional (name has '.') */
+    size_t att_mch_fst=0L; /* [nbr] Number of characters by which to offset attribute string comparison */
+    size_t obj_mch_fst=0L; /* [nbr] Number of characters by which to offset object string comparison */
 
-    /* Rename attribute of single variable... */
-    if(strchr(att_rnm_lst[idx_att].old_nm,dlm_chr)){
-      /* Extract variable name from old name */
-      rcd_att=nco_prs_att((att_rnm_lst+idx_att),var_nm,&IS_GLB_GRP_ATT);
-      if(!rcd_att){
-        (void)fprintf(stdout,"%s: ERROR Could not parse var_nm@att_nm string \"%s\"\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm);
-        nco_exit(EXIT_FAILURE);
-      } /* end if */ 
+    /* Extract variable name from old name */
+    rcd_att=nco_prs_att((att_rnm_lst+idx_att),obj_nm,&mch_grp_all,&mch_grp_glb,&mch_obj_all);
+    if(!rcd_att){
+      (void)fprintf(stdout,"%s: ERROR Could not parse obj_nm@att_nm string \"%s\"\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm);
+      nco_exit(EXIT_FAILURE);
+    } /* end if */ 
 
-      /* Inquire if any object matches "var_nm" */
-      trv_obj=nco_obj_usr_sng(var_nm,trv_tbl,&is_opt);  
+    if(obj_nm[0] == opt_chr){
+      obj_is_opt=True;
+      obj_mch_fst=1L;
+    } /* end if */
 
-      /* If object is group, set NC_GLOBAL */
-      if(trv_obj && trv_obj->nco_typ == nco_obj_typ_grp) var_id=NC_GLOBAL;
+    if(att_rnm_lst[idx_att].old_nm[0] == opt_chr){
+      att_is_opt=True;
+      att_mch_fst=1L;
+    } /* end if */
 
-      /* Object found that matches "var_nm" */
-      if(trv_obj || IS_GLB_GRP_ATT){
+    /* Be forgiving */
+    if(mch_obj_all) obj_is_opt=True;
 
-        /* If object found get group ID, else groud ID is root (cases of "global") */
-        if(trv_obj) (void)nco_inq_grp_full_ncid(nc_id,trv_obj->grp_nm_fll,&grp_id); else grp_id=nc_id;
+    if(nco_dbg_lvl_get() >= nco_dbg_io) (void)fprintf(stdout,"%s: att_rnm_lst[].old_nm = \"%s\", obj_nm = \"%s\", att_is_opt = %d, att_mch_fst = %ld, obj_is_opt = %d, obj_mch_fst = %ld, mch_fll_nm = %d, mch_grp_all = %d, mch_grp_glb = %d, mch_obj_all = %d\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm,obj_nm,att_is_opt,att_mch_fst,obj_is_opt,obj_mch_fst,mch_fll_nm,mch_grp_all,mch_grp_glb,mch_obj_all);
 
-        /* Get var_id of variable */
-        if(IS_GLB_GRP_ATT){
-          (void)fprintf(stdout,"%s: INFO Assuming \"%s\" refers to a Global or Group attribute\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm);
-          var_id=NC_GLOBAL;
-        }else{ /* !IS_GLB_GRP_ATT */
-          if(var_nm[0] == opt_chr){
-            rcd=nco_inq_varid_flg(grp_id,var_nm+1L,&var_id);
-            if(rcd != NC_NOERR){
-              (void)fprintf(stdout,"%s: WARNING Variable \'%s\' not present in %s, skipping it.\n",nco_prg_nm,var_nm+1L,fl_in);
-              /* Reset error code */
-              rcd=NC_NOERR; 
-              /* Optional variable not found, continue to next attribute in list */
-              continue;
-            } /* end if */
-          }else{ /* Variable name does not contain opt_chr so variable presence is required */
+    /* Compare each attribute in user list to each attribute name in file */
+    for(unsigned int tbl_idx=0;tbl_idx<trv_tbl->nbr;tbl_idx++){
+      nco_bool obj_is_var=False; /* [flg] Object with attribute is variable */
+      nco_bool mch_obj_crr=False; /* [flg] User-specified object matches current object */
+      
+      /* Global group only */
+      if(mch_grp_glb && trv_tbl->lst[tbl_idx].nm_fll_lng == 1L){
+	mch_obj_crr=True;
+	mch_fll_nm=True;
+      } /* endif */
 
-            /* Get ID only if object is variable (not group). NB: use relative name found */
-            if(trv_obj->nco_typ == nco_obj_typ_var) rcd=nco_inq_varid(grp_id,trv_obj->nm,&var_id);
+      /* Match all groups */
+      if(!mch_obj_crr && trv_tbl->lst[tbl_idx].nco_typ == nco_obj_typ_grp && mch_grp_all) mch_obj_crr=True;
 
-          } /* end if */
-        } /* !IS_GLB_GRP_ATT */
-        if(rcd == NC_NOERR){
-          if(att_rnm_lst[idx_att].old_nm[0] == opt_chr){
-            /* Preceding opt_chr means attribute need not be present */
-            rcd=nco_inq_attid_flg(grp_id,var_id,att_rnm_lst[idx_att].old_nm+1L,&att_rnm_lst[idx_att].id);	  
-            if(rcd == NC_NOERR){
-              (void)nco_rename_att(grp_id,var_id,att_rnm_lst[idx_att].old_nm+1L,att_rnm_lst[idx_att].new_nm);
-              nbr_rnm++;
-              if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm+1L,att_rnm_lst[idx_att].new_nm,(var_nm[0] == opt_chr ? var_nm+1L : var_nm));
-            }else{
-              (void)fprintf(stdout,"%s: WARNING Attribute \'%s\' not present in variable \'%s\'\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm+1L,(var_nm[0] == opt_chr ? var_nm+1L : var_nm));
-            } /* endelse */
-          }else{ 
-            /* Attribute must be present */
-            rcd=nco_inq_attid(grp_id,var_id,att_rnm_lst[idx_att].old_nm,&att_rnm_lst[idx_att].id);
-            if(rcd == NC_NOERR){
-              (void)nco_rename_att(grp_id,var_id,att_rnm_lst[idx_att].old_nm,att_rnm_lst[idx_att].new_nm);
-              nbr_rnm++;
-              if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm,att_rnm_lst[idx_att].new_nm,(var_nm[0] == opt_chr ? var_nm+1L : var_nm));
-            } /* endif attribute is present */
-          } /* endelse attribute must be present */  
-        }else{ /* variable not present */
-          (void)fprintf(stdout,"%s: WARNING variable \'%s\' not present in %s\n",nco_prg_nm,var_nm,fl_in);
-        }  /* variable not present */
-        /* end if renaming single variable */
+      /* Match all objects */
+      if(!mch_obj_crr && mch_obj_all) mch_obj_crr=True;
 
-      } /* Match variable by name */
-
-    }else{ /* ...or rename attribute for all variables and groups... */
-
-      for(unsigned int idx_tbl=0;idx_tbl<trv_tbl->nbr;idx_tbl++){
-        (void)nco_inq_grp_full_ncid(nc_id,trv_tbl->lst[idx_tbl].grp_nm_fll,&grp_id);
-
-        if(trv_tbl->lst[idx_tbl].nco_typ == nco_obj_typ_grp) var_id=NC_GLOBAL; else (void)nco_inq_varid(grp_id,trv_tbl->lst[idx_tbl].nm,&var_id);
-
-        if(att_rnm_lst[idx_att].old_nm[0] == opt_chr){
-          /* Rename attribute if variable/group contains attribute else do nothing */
-          rcd=nco_inq_attid_flg(grp_id,var_id,att_rnm_lst[idx_att].old_nm+1L,&att_rnm_lst[idx_att].id);
-          if(rcd == NC_NOERR){
-            (void)nco_rename_att(grp_id,var_id,att_rnm_lst[idx_att].old_nm+1L,att_rnm_lst[idx_att].new_nm);
-            nbr_rnm++;
-            /* Inform user which variable had attribute renamed */
-            if(var_id > NC_GLOBAL){
-              if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm+1L,att_rnm_lst[idx_att].new_nm,trv_tbl->lst[idx_tbl].nm);
-            }else{
-              if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed global or group attribute \'%s\' to \'%s\'\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm+1L,att_rnm_lst[idx_att].new_nm);
-            } /* end else */
-          }else{ /* end if attribute was found */
-            /* Reset error code */
-            rcd=NC_NOERR; 
-          } /* end else */
-        }else{ /* !opt_chr */
-          /* Rename attribute or die trying */
-          rcd=nco_inq_attid_flg(grp_id,var_id,att_rnm_lst[idx_att].old_nm,&att_rnm_lst[idx_att].id);
-          if(rcd == NC_NOERR){
-            (void)nco_rename_att(grp_id,var_id,att_rnm_lst[idx_att].old_nm,att_rnm_lst[idx_att].new_nm);
-            nbr_rnm++;
-            /* Inform user which variable had attribute renamed */
-            if(var_id > NC_GLOBAL){
-              if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed attribute \'%s\' to \'%s\' for variable \'%s\'\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm,att_rnm_lst[idx_att].new_nm,var_nm);
-            }else{
-              IS_GLB_GRP_ATT=True; /* [flg] Attribute is Global or Group attribute */
-              if(IS_GLB_GRP_ATT) (void)fprintf(stdout,"%s: INFO found and renamed global or group attribute \'%s\' so not requiring its presence in every variable.\n",nco_prg_nm_get(),att_rnm_lst[idx_att].old_nm); 
-              if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed global or group attribute \'%s\' to \'%s\'\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm,att_rnm_lst[idx_att].new_nm);
-            } /* end else */
-          }else{ /* !NC_NOERR */
-            /* Reset error code or print informative message and die */
-            if(IS_GLB_GRP_ATT){
-              /* Forgive omission of period for global/group attributes. Users aren't perfect :) */
-              rcd=NC_NOERR;
-            }else{ /* !IS_GLB_GRP_ATT */
-              (void)fprintf(stdout,"%s: ERROR User specified that presence of attribute \'%s\' is required. However, the %s%s does not contain it. HINT: If attribute presence is intended to be optional, then prefix attribute name with the period character \'%c\', e.g., %catt_nm. With this syntax %s would succeed even if no variables or groups contained the attribute. If attribute is intended to be renamed only in a specific variable, then prepend the variable name plus an at-sign \'%c\' to the attribute name, e.g., var_nm%catt_nm. If attribute presence is required only for global or group attributes, then prefix attribute name with \"global\" and an at-sign, e.g., global%catt_nm.\n",nco_prg_nm_get(),att_rnm_lst[idx_att].old_nm,(var_id > NC_GLOBAL) ? "variable " : "root group",(var_id > NC_GLOBAL) ? var_nm : "",opt_chr,opt_chr,nco_prg_nm_get(),dlm_chr,dlm_chr,dlm_chr);
-              /* Exit now rather than completing variable loop and printing lengthy error message above each iteration */
-              nco_err_exit(rcd,"main");
-            } /* !IS_GLB_GRP_ATT */
-          } /* !NC_NOERR */
-        } /* !opt_chr */
-      } /* Loop table */
-    } /* end if renaming attribute for all variables */
-    /* See to it that any mandatory renaming was performed, else abort */
-    if(nbr_rnm == 0){
-      if(att_rnm_lst[idx_att].old_nm[0] == opt_chr){
-        (void)fprintf(stdout,"%s: WARNING Attribute \'%s\' not renamed because not found in searched variable(s)\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm+1L);
-      }else{
-        (void)fprintf(stdout,"%s: ERROR Attribute \'%s\' not present in %s, aborting.\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm,fl_in);
-        nco_exit(EXIT_FAILURE);
+      /* Specific object name is provided so test for match */
+      if(!mch_obj_crr && obj_nm){
+	if(!strcmp(obj_nm+obj_mch_fst,trv_tbl->lst[tbl_idx].nm_fll)){
+	  mch_obj_crr=True;
+	  mch_fll_nm=True;
+	}else if(!strcmp(obj_nm+obj_mch_fst,trv_tbl->lst[tbl_idx].nm)){
+	  mch_obj_crr=True;
+	} /* endif strcmp() */
+	if(mch_obj_crr && trv_tbl->lst[tbl_idx].nco_typ == nco_obj_typ_grp) (void)fprintf(stdout,"%s: INFO Assuming \"%s\" refers to a Global or Group attribute\n",nco_prg_nm,obj_nm+obj_mch_fst);
       } /* end else */
-    } /* nbr_rnm */
-  } /* end loop over attributes to rename */
+
+      if(mch_obj_crr){
+	/* Obtain group and/or variable ID for current object match */
+	(void)nco_inq_grp_full_ncid(nc_id,trv_tbl->lst[tbl_idx].grp_nm_fll,&grp_id);
+	if(trv_tbl->lst[tbl_idx].nco_typ == nco_obj_typ_var){
+	  obj_is_var=True;
+	  rcd=nco_inq_varid(grp_id,trv_tbl->lst[tbl_idx].nm,&var_id);
+	}else{
+	  var_id=NC_GLOBAL;
+	} /* endelse */
+      } /* !mch_obj_crr */
+
+      if(mch_obj_crr){
+	/* Locate attribute */
+	rcd=nco_inq_attid_flg(grp_id,var_id,att_rnm_lst[idx_att].old_nm+att_mch_fst,&att_rnm_lst[idx_att].id); 
+	if(att_is_opt && rcd != NC_NOERR){
+	  if(nco_dbg_lvl_get() >= nco_dbg_var) (void)fprintf(stdout,"%s: INFO Optional attribute \'%s\' not present in %s \'%s\', skipping it\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm+att_mch_fst,obj_is_var ? "variable" : "group",trv_tbl->lst[tbl_idx].nm_fll);
+	  continue;
+	} /* endif optional attribute */
+	if(!att_is_opt && rcd != NC_NOERR){
+	  (void)fprintf(stdout,"%s: ERROR Required attribute \'%s\' not present in %s \'%s\'.\nHINT: If attribute presence is intended to be optional, then prefix attribute name with the period character \'%c\', e.g., %c%s. With this syntax %s would succeed even if no variables or groups contained the attribute. If attribute is intended to be renamed only in a specific variable, then prepend the variable name plus an at-sign \'%c\' to the attribute name, e.g., var_nm%catt_nm. If attribute presence is required only for root group (i.e., a global attribute), then prefix attribute name with \"global\" and an at-sign, e.g., global%catt_nm. If attribute presence is required for all groups, then prefix attribute name with \"group\" and an at-sign, e.g., group%catt_nm.\n",nco_prg_nm_get(),att_rnm_lst[idx_att].old_nm,obj_is_var ? "variable" : "group",trv_tbl->lst[tbl_idx].nm_fll,opt_chr,opt_chr,att_rnm_lst[idx_att].old_nm,nco_prg_nm_get(),dlm_chr,dlm_chr,dlm_chr,dlm_chr);
+	  nco_err_exit(rcd,"main");
+	} /* endif required attribute */
+
+	(void)nco_rename_att(grp_id,var_id,att_rnm_lst[idx_att].old_nm+att_mch_fst,att_rnm_lst[idx_att].new_nm);
+	mch_nbr++;
+	if(nco_dbg_lvl >= nco_dbg_fl) (void)fprintf(stdout,"%s: Renamed attribute \'%s\' to \'%s\' for %s \'%s\'\n",nco_prg_nm,att_rnm_lst[idx_att].old_nm+att_mch_fst,att_rnm_lst[idx_att].new_nm,obj_is_var ? "variable" : "group",trv_tbl->lst[tbl_idx].nm_fll);
+
+      } /* endif found object */
+
+      /* Matched full name for uniquely specified attribute, move-on to next rename request, i.e.,
+	 do not bother examining remainder of traversal table for another match */
+      if(mch_fll_nm) break;
+
+    } /* end loop over traversal table */
+    
+    if(mch_nbr == 0){
+      /* Object was not found in traversal table */
+      if(!mch_obj_all && obj_is_opt && obj_nm){
+	(void)fprintf(stdout,"%s: INFO Optional object \'%s\' not present in %s, skipping it\n",nco_prg_nm,obj_nm+obj_mch_fst,fl_in);
+	/* Optional object not found, continue to next attribute in list */
+	continue;
+      }else if(!obj_is_opt && obj_nm){
+	(void)fprintf(stdout,"%s: ERROR Required object \'%s\' not present in %s\n",nco_prg_nm,obj_nm+obj_mch_fst,fl_in);
+	nco_err_exit(rcd,"main");
+      } /* !obj_is_opt */
+    } /* endif specified object not found */
+
+    if(nco_dbg_lvl >= nco_dbg_std) (void)fprintf(stdout,"%s: Renamed %d attribute%s from \'%s\' to \'%s\'\n",nco_prg_nm,mch_nbr,mch_nbr != 1 ? "s" : "",att_rnm_lst[idx_att].old_nm,att_rnm_lst[idx_att].new_nm);
+
+    mch_nbr_att+=mch_nbr;
+
+  } /* end attributes */
+
+  /* Reset error return code */
+  rcd=NC_NOERR;
+
+  (void)fprintf(stdout,"%s: In total renamed ",nco_prg_nm);
+  (void)fprintf(stdout,"%d attribute%s",mch_nbr_att,mch_nbr_att != 1 ? "s" : "");
+  (void)fprintf(stdout,", %d dimension%s",mch_nbr_dmn,mch_nbr_dmn != 1 ? "s" : "");
+  (void)fprintf(stdout,", %d group%s",mch_nbr_grp,mch_nbr_grp != 1 ? "s" : "");
+  (void)fprintf(stdout,", and %d variable%s\n",mch_nbr_var,mch_nbr_var != 1 ? "s" : "");
 
   /* Catenate time-stamped command line to "history" global attribute */
   if(HISTORY_APPEND) (void)nco_hst_att_cat(nc_id,cmd_ln);
@@ -629,6 +639,7 @@ main(int argc,char **argv)
     if(nbr_dmn_rnm > 0) dmn_rnm_lst=(rnm_sct *)nco_free(dmn_rnm_lst);
     if(nbr_grp_rnm > 0) grp_rnm_lst=(rnm_sct *)nco_free(grp_rnm_lst);
     if(nbr_var_rnm > 0) var_rnm_lst=(rnm_sct *)nco_free(var_rnm_lst);
+    if(obj_nm) obj_nm=(char *)nco_free(obj_nm);
 
     /* NCO-generic clean-up */
     /* Free individual strings/arrays */
